# Parallel Cyclostationary Feature Extraction Algorithm

## Introduction

Cyclostationary feature detection is one of the most common methods used in spectrum sensing. The parallel algorithm presented here can be used in multi-core processors to speedup the execution time. The implementation uses OpenMP and the Fastest Fourier Transform in the West (a efficient Fourier transform method).

## Dependencies

* libfftw3-dev (mandatory):  _The FFTW library computes Fast Fourier Transforms (FFT) in one or more dimensions. It is extremely fast. This package contains the statically linked  library, header files and test programs._
* libgomp1 (mandatory): _GCC OpenMP (GOMP) support library  GOMP is an implementation of OpenMP for the C, C++, and Fortran compilers  in the GNU Compiler Collection._
* gnuplot (optional):  _Gnuplot is a portable command-line driven interactive data and function plotting utility that supports lots of output formats, including drivers  for many printers, (La)TeX, (x)fig, Postscript, and so on._

## Description of the files

* Code/pcpd.c: This is the main program
* Code/mylib.c: Some routines to dynamically allocate the data structures and generate a random (uniformly distributed) signal
* Code/load_signal.c: Load the signal that will be analyzed
* Code/save_signal.c: Save the cyclostationary features in two files: Results/S_re.txt (real part) and Results/S_im.txt (imaginary part)
* Code/load_Sre_Sim.m: This matlab script can be used to plot the cyclostationary features stored in the files _Results/S_re.txt_ (real part) and _Results/S_im.txt_ (imaginary part)
* Data/bpsk.txt: a BPSK modulated sinal that can be used to test the cyclostationary feature extraction
* Results/results.txt: stores the execution results: median time, number oh threads used, number of cores avalilable, block size (N), number of blocks (L), and number of iterations used in Monte Carlo simulations (n_loops)
* Results/S_re.txt: Real part of the cyclostationary features
* Results/S_im.txt: Imaginary part of the cyclostationary features

## Compiling the program

gcc pcpd.c -o pcpd -std=c99 -lm -fopenmp -lfftw3f_omp -lfftw3f

## Execution instructions

* The number of threads in use can be defined setting the following variable in shell (e.g., to use 4 threads): __export OMP_NUM_THREADS=4__
* Note that Code Ocean has only 4 physical cores available (8 with hyperthreading)
* By default, the program will generate a random signal and extract the cyclostationary features. The aim is only measure the execution time to calculate the parallel performance. If you want to analyze a signal stored in a file uncomment the lines:
```C
//int len2 = 0;
//float \_Complex* x = load_signal(&len2); // input signal
```
and comment this one:
```C
float _Complex* x = gen_rand_sig( len );
```
