// Dynamically allocate 2 dimensional array (x[nrows][ncolumns])
float _Complex** alloc2d(const int nrows, const int ncolumns){
	float _Complex **x;
	x = malloc( nrows * sizeof(float _Complex *) );
	if(x==NULL){
		fprintf(stderr, "out of memory (x[])\n");
		exit(-1);
	}
	
	for(int l=0; l<nrows; l++){
		x[l] = malloc( ncolumns * sizeof (float _Complex) );
		if(x[l]==NULL){
			fprintf(stderr, "out of memory (x[%d][N])\n",l);
			exit(-1);
		}
	}
	return x;
}

// Dynamically allocate 1 dimensional array (x[nrows])
float _Complex* alloc1d(const int nrows){
	float _Complex *x;
	x = malloc( nrows * sizeof(float _Complex) );
	if(x==NULL){
		fprintf(stderr, "out of memory (x[])\n");
		exit(-1);
	}	
	return x;
}

float _Complex** alloc1d_p(const int nrows){
	float _Complex **xl;
	xl = malloc( nrows * sizeof(float _Complex *) );
	if(xl==NULL){
		fprintf(stderr, "out of memory (xl[])\n");
		exit(-1);
	}
	return xl;
}

void free2d(float _Complex **v, const int nrows){
	for(int n=0; n<nrows; n++){
		free( v[n] );
	}
	free(v); v=NULL;
}

int T1_offset(const int alpha, const int N){ 
	if(alpha>N){
		fprintf(stderr, "Error: alpha should be less or equal than N\n");
		exit(-1);
	}
	return ( alpha * (2*N - alpha + 1) ) / 2;
}

int float_cmp(const void* a,const void * b){
	const float *ia = (const float *)a; // casting pointer types 
	const float *ib = (const float *)b;
	if ( *ia < *ib ){
		return -1;
	}else{
		if ( *ia > *ib ){
			return 1;
		}else{
			return 0;
		}
	}
}

float _Complex* gen_rand_sig(const int len){
	float _Complex *x;
	x = malloc( len * sizeof(float _Complex) );
	if(x==NULL){
		fprintf(stderr, "out of memory (x[])\n");
		exit(-1);
	}
	srand( time(NULL) );
	for(int i=0; i<len; i++){
		x[i] = (rand() + rand()*_Complex_I)/RAND_MAX;
	}
	return x;
}
